#ifndef __DISCOVER_PROTO_H__
#define __DISCOVER_PROTO_H__

/*
********* DISCOVER DATAGRAM **********
	DISCOVER_REQ
		message type : 4byte
		client mac   : 8byte
	DISCOVER_RES
		message type : 4byte
		client mac   : 8byte
		server mac   : 8byte
                use dhcp     : 4byte
		ip address   : 4byte
		broad cast   : 4byte
                gate way     : 4byte
	IP_OFFER_REQ
		message type : 4byte
		client mac   : 8byte
		server mac   : 8byte
                use dhcp     : 4byte
		ip address   : 4byte
		broad cast   : 4byte
                gate way     : 4byte
	IP_OFFER_RES
		message type : 4byte
		client mac   : 8byte
		server mac   : 8byte
                use dhcp     : 4byte
		ip address   : 4byte
		broad cast   : 4byte
                gate way     : 4byte
*/
		
#include <stdint.h>
#include <syslog.h>

#define SERVER_PORT	22222
#define CLIENT_PORT	22223

/* debug macro */
#ifdef __DEBUG__
#define LOG(args...)    ::printf(args)
#define ERR(args...)    ::printf(args)
#define PERROR			perror
#else
#define LOG(args...)    {}
#define ERR(args...)    ::printf(args)
#define PERROR			{}
#endif

typedef enum result
{
	SUCCESS = 0,
	SET_IP_ADDRESS_FAILED = 1,
	SET_SUBNETMASK_FAILED = 2,
	SET_BROAD_CAST_FAILED = 3,
	SET_DEFAULT_GW_FAILED = 4,
}result_e;

enum message_type_e
{
	DISCOVER_REQ = 0,
	DISCOVER_REP = 1,
	IP_OFFER_REQ = 2,
	IP_OFFER_REP = 3,	
	INVALID_MESSAGE_TYPE = 4,
};

struct network_info_s
{
    uint32_t use_dhcp;
    uint32_t ip_address;
    uint32_t netmask;
    uint32_t gateway;
};

struct discover_packet_s
{
	uint32_t message_type;
	uint32_t transaction_id;
    char mac_addr[8];
	uint32_t result;
	network_info_s net_info;
};

class CDiscover
{
public:
	CDiscover();
	~CDiscover();
	bool Setup();
	uint32_t SendFindPacket();
	uint32_t SendConfigPacket(
					const char* macaddr,
					const network_info_s* networkInfo
					);
	bool RecvReplyPacket(
					discover_packet_s* packet,
					uint32_t transactionID,
					unsigned int timeout = 0
					);

protected:
	bool RecvPacket(discover_packet_s* packet);
	bool SendPacket(const discover_packet_s* packet);
	uint32_t GetNewTransactionID() const;

private:
	int m_sock;
	struct sockaddr_in m_sockaddr_broadcast;
};

#endif
